@@ -190,24 +190,10 @@ def expand(abbreviation: str, country_code: Optional[str] = None) -> List[db.Geo
     return result
 
 
-def contains(large: Union[db.Geoname, db.CountryInfo], small: db.Geoname) -> bool:
-    if isinstance(large, db.CountryInfo):
-        return large.iso == small.country_code
-    elif small.country_code != large.country_code:
-        return False
-
-    if large.feature_code == 'PCLI':
-        return True
-    elif large.feature_code.startswith('ADM1'):
-        return small.admin1_code == large.admin1_code
-    elif large.feature_code.startswith('ADM2'):
-        return small.admin2_code == large.admin2_code
-    elif large.feature_code.startswith('ADM3'):
-        return small.admin3_code == large.admin3_code
-    elif large.feature_code.startswith('ADM4'):
-        return small.admin4_code == large.admin4_code
-
-    return False
+def _contains(large: db.Geoname, small: db.Geoname) -> bool:
+    large_code = '.'.join(v for (n, v) in _generate_admin_codes(large))
+    small_code = '.'.join(v for (n, v) in _generate_admin_codes(small))
+    return small_code.startswith(large_code)
 
 
 def _match(
@@ -216,7 +202,7 @@ def _match(
 ) -> Iterable[Tuple[db.Geoname, db.Geoname]]:
     for s in states:
         for c in cities:
-            if contains(s, c):
+            if _contains(s, c):
                 yield c, s
 
 
@@ -304,11 +290,14 @@ def sc_list(
     >>> [g.name for g in sc_list('or', 'united states')]
     ['Oregon']
     """
+    assert db.TRIE
+    try:
+        params = [t[2] for t in db.TRIE[state.lower()]]
+    except KeyError:
+        return None
+
     buf = io.StringIO()
     buf.write('WHERE feature_code in ("ADM1", "ADM2", "ADM3") ')
-
-    assert db.TRIE
-    params = [t[2] for t in db.TRIE[state.lower()]]
     buf.write('AND geonameid IN (%s) ' % ','.join(['?' for _ in params]))
 
     if country:
@@ -336,6 +325,45 @@ def sc_list(
     return db.select_geonames(subcommand, params)
 
 
+def _generate_admin_codes(
+    geoname: db.Geoname,
+    max_level: Optional[int] = None,
+) -> Iterator[Tuple[str, str]]:
+    if max_level is None:
+        if geoname.feature_code == 'PCLI':
+            max_level = 0
+        elif geoname.feature_code == 'ADM1':
+            max_level = 1
+        elif geoname.feature_code == 'ADM2':
+            max_level = 2
+        elif geoname.feature_code == 'ADM3':
+            max_level = 3
+        else:
+            max_level = 4
+
+    assert max_level is not None
+    yield 'country_code', geoname.country_code
+
+    for level in (1, 2, 3, 4):
+        if level > max_level:
+            break
+        field_name = 'admin%d_code' % level
+        field_value = getattr(geoname, field_name)
+        if field_value:
+            yield field_name, field_value
+
+
+def _wrap(geoname: db.Geoname) -> Union['Geoname', 'Country', 'State', 'City']:
+    if geoname.feature_code == 'PCLI':
+        return Country(geoname.country_code)
+    elif geoname.feature_class == 'A':
+        return State(geoname)
+    elif geoname.feature_class == 'P':
+        return City(geoname)
+    else:
+        return Geoname(geoname)
+
+
 #
 # Wraps the db.Geoname namedtuple to provide additional behavior.
 #
@@ -345,11 +373,21 @@ class Geoname:
     Every place name in the ``geonames.org`` dataset, including countries,
     states, etc. corresponds to one of these.
 
-    Corresponds directly to the data model described at 
+    Corresponds directly to the data model described at
     http://download.geonames.org/export/dump/.
     """
     def __init__(self, geoname: db.Geoname) -> None:
         self.geoname = geoname
+        self._cursor = None
+        self._params = [self.geoname.geonameid]
+
+        buf = io.StringIO()
+        buf.write('SELECT * FROM geoname WHERE geonameid != ? ')
+        for field_name, field_value in _generate_admin_codes(self.geoname):
+            buf.write('AND %s = ? ' % field_name)
+            self._params.append(field_value)
+        buf.write('ORDER BY population DESC')
+        self._command = buf.getvalue()
 
     @property
     def geonameid(self) -> int:
@@ -441,24 +479,12 @@ class Geoname:
         """date of last modification in yyyy-MM-dd format"""
         return self.geoname.modification_date
 
-    def __contains__(self, other):
-        if isinstance(other, str):
-            for g in db.select_geonames_name(other):
-                if contains(self.geoname, g):
-                    return True
-        elif isinstance(other, Geoname):
-            return contains(self.geoname, other.geoname)
-        elif isinstance(other, db.Geoname):
-            return contains(self.geoname, other)
-        return False
-
     def normalize(self, language: str = DEFAULT_LANGUAGE) -> str:
         """Returns the preferred name of this place."""
-        c = db.CONN.cursor()
-
-        canonical = db.select_geonames_ids([self.geonameid])[0].name
         if language == 'en':
-            return canonical
+            return self.geoname.name
+
+        c = db.CONN.cursor()
 
         def _lookup(subcommand):
             command = (
@@ -482,7 +508,7 @@ class Geoname:
         if alt:
             return alt
 
-        return canonical
+        return self.geoname.name
 
     @property
     def country(self) -> 'Country':
@@ -526,25 +552,27 @@ class Geoname:
         return db.Geoname(*next(result))
 
     @property
-    def admin1(self) -> 'State':
+    def admin1(self) -> Optional['State']:
         """Navigate to the first level of the administrative hierarchy.
 
         >>> Country('us').cities['boston'].admin1.name
         'Massachusetts'
         """
         if self.feature_code == 'PCLI':
-            return StateCollection(self, 'ADM1')
+            return Collection(self, feature_code='ADM1')
         elif self.feature_code == 'ADM1':
             return self
-        else:
+        elif self.admin1_code:
             return State(self._get_parent_adm(1))
+        else:
+            return None
 
     @property
-    def admin2(self) -> Union['State', 'StateCollection']:
+    def admin2(self) -> Optional[Union['State', 'Collection']]:
         """Navigate to the second level of the administrative hierarchy.
 
         For entities that exist above that level, this will be a collection
-        of second-level descendants: 
+        of second-level descendants:
 
         >>> [a2.name for a2 in Country('us').admin1['ma'].admin2][:3]
         ['Middlesex County', 'Worcester County', 'Essex County']
@@ -556,18 +584,20 @@ class Geoname:
         'Suffolk County'
         """
         if self.feature_code in ('PCLI', 'ADM1'):
-            return StateCollection(self, 'ADM2')
+            return Collection(self, feature_code='ADM2')
         elif self.feature_code == 'ADM2':
             return self
-        else:
+        elif self.admin2_code:
             return State(self._get_parent_adm(2))
+        else:
+            return None
 
     @property
-    def admin3(self) -> Union['State', 'StateCollection']:
+    def admin3(self) -> Optional[Union['State', 'Collection']]:
         """Navigate to the third level of the administrative hierarchy.
 
         For entities that exist above that level, this will be a collection
-        of third-level descendants: 
+        of third-level descendants:
 
         >>> [a3.name for a3 in Country('us').admin1['ma'].admin2['suffolk'].admin3][:3]
         ['City of Boston', 'City of Revere', 'City of Chelsea']
@@ -579,30 +609,83 @@ class Geoname:
         'City of Boston'
         """
         if self.feature_code in ('PCLI', 'ADM1', 'ADM2'):
-            return StateCollection(self, 'ADM3')
+            return Collection(self, feature_code='ADM3')
         elif self.feature_code == 'ADM3':
             return self
-        else:
+        elif self.admin3_code:
             return State(self._get_parent_adm(3))
-
+        else:
+            return None
 
     @property
-    def admin4(self) -> Union['State', 'StateCollection']:
+    def admin4(self) -> Optional[Union['State', 'Collection']]:
         if self.feature_code in ('PCLI', 'ADM1', 'ADM2', 'ADM3'):
-            return StateCollection(self, 'ADM4')
+            return Collection(self, feature_code='ADM4')
         elif self.feature_code == 'ADM4':
             return self
-        else:
+        elif self.admin4_code:
             return State(self._get_parent_adm(4))
+        else:
+            return None
 
     @property
     def state(self):
         return self.admin1
 
+    @property
+    def full_admin_code(self) -> str:
+        values = [value for (unused_name, value) in _generate_admin_codes(self.geoname)]
+        return '.'.join(values)
+
     def distance_to(self, other: 'Geoname') -> float:
         """Returns the distance to the other place."""
         return _haversine_dist(self.latitude, self.longitude, other.latitude, other.longitude)
 
+    def __repr__(self):
+        return 'Geoname.gid(%r, %r, %r)' % (self.geonameid, self.name, self.country_code)
+
+    def __str__(self):
+        return 'Geoname(%r, %r)' % (self.name, self.country_code)
+
+    def __iter__(self):
+        return self
+
+    def __next__(self) -> db.Geoname:
+        if self._cursor is None:
+            self._cursor = db.CONN.cursor()
+            self._cursor.execute(self._command, self._params)
+
+        while True:
+            result = next(self._cursor)
+            geoname = db.Geoname(*result)
+            if _contains(self.geoname, geoname):
+                return _wrap(geoname)
+
+    def __len__(self):
+        cursor = db.CONN.cursor()
+        command = self._command.replace('SELECT *', 'SELECT COUNT(*)')
+        return next(cursor.execute(command, self._params))[0]
+
+    def __getitem__(self, key):
+        for geoname in db.select_geonames_name(key):
+            if self.geoname != geoname and _contains(self.geoname, geoname):
+                return _wrap(geoname)
+        raise KeyError('could not find %r in %r' % (key, self))
+
+    def __contains__(self, other):
+        if isinstance(other, str):
+            try:
+                self[other]
+            except KeyError:
+                return False
+            else:
+                return True
+        elif isinstance(other, Geoname):
+            return _contains(self.geoname, other.geoname)
+        elif isinstance(other, db.Geoname):
+            return _contains(self.geoname, other)
+        return False
+
 
 #
 # Wraps both the db.Geoname and db.CountryInfo namedtuples and provides
@@ -634,7 +717,7 @@ class Country(Geoname):
 
     @property
     def iso3(self) -> str:
-        return self.info.iso2
+        return self.info.iso3
 
     @property
     def iso_numeric(self) -> int:
@@ -653,9 +736,10 @@ class Country(Geoname):
         c = db.CONN.cursor()
         command = (
             'SELECT * FROM geoname '
-            'WHERE name = ? AND country_code = ? AND feature_code = "PPLC"'
+            'WHERE (name = ? OR asciiname = ?) '
+            'AND country_code = ? AND feature_code = "PPLC"'
         )
-        result = c.execute(command, (self.info.capital, self.iso))
+        result = c.execute(command, (self.info.capital, self.info.capital, self.iso))
         return City(db.Geoname(*next(result)))
 
     @property
@@ -713,7 +797,7 @@ class Country(Geoname):
             return item.country_code == self.country_code
         elif isinstance(item, str):
             for g in db.select_geonames_name(item):
-                if contains(self.geoname, g):
+                if _contains(self.geoname, g):
                     return True
         return False
 
@@ -721,121 +805,117 @@ class Country(Geoname):
         return expand(abbreviation, country_code=self.iso)
 
     @property
-    def states(self) -> 'StateCollection':
-        return StateCollection(parent=self, feature_code=self._state_feature_code)
+    def states(self) -> 'Collection':
+        return Collection(parent=self, feature_code=self._state_feature_code)
 
     @property
-    def admin1(self) -> 'StateCollection':
-        return StateCollection(parent=self, feature_code='ADM1')
+    def admin1(self) -> 'Collection':
+        return Collection(parent=self, feature_code='ADM1')
 
     @property
-    def admin2(self) -> 'StateCollection':
-        return StateCollection(parent=self, feature_code='ADM2')
+    def admin2(self) -> 'Collection':
+        return Collection(parent=self, feature_code='ADM2')
 
     @property
-    def admind(self) -> 'StateCollection':
-        return StateCollection(parent=self, feature_code='ADMD')
+    def admind(self) -> 'Collection':
+        return Collection(parent=self, feature_code='ADMD')
 
     @property
-    def cities(self) -> 'CityCollection':
-        return CityCollection(parent=self)
+    def cities(self) -> 'Collection':
+        return Collection(parent=self, feature_class='P')
+
+
+COUNTRIES = [Country(i.iso) for i in db.COUNTRYINFO]
 
 
 class Collection:
     """Represents an abstract collection of places."""
     def __init__(
         self,
-        parent: Geoname,
-        feature_class: str,
-        feature_code: Optional[str],
+        parent: Optional[Geoname],
+        feature_class: Optional[str] = None,
+        feature_code: Optional[str] = None,
     ) -> None:
         self._parent = parent
         self._feature_class = feature_class
         self._feature_code = feature_code
 
-        self._params = [self._parent.country_code]
-
-        self._cursor = db.CONN.cursor()
+        self._params = []
+        self._cursor = None
 
         buf = io.StringIO()
         buf.write('SELECT * FROM geoname ')
-        buf.write('WHERE country_code = ? ')
-        if feature_code:
-            buf.write('AND feature_code = ? ')
-            self._params.append(feature_code)
-        else:
-            buf.write('AND feature_class = ? ')
-            self._params.append(feature_class)
 
-        for field_name, field_value in self._generate_admin_codes():
-            buf.write('AND %s = ?' % field_name)
-            self._params.append(field_value)
+        def generate_constraints():
+            if feature_code:
+                yield 'feature_code', feature_code
+            if feature_class:
+                yield 'feature_class', feature_class
+            if self._parent:
+                for name, value in _generate_admin_codes(self._parent.geoname):
+                    yield name, value
 
-        self._command = buf.getvalue()
+        constraints = list(generate_constraints())
+        if constraints:
+            field_names, self._params = zip(*constraints)
+            buf.write('WHERE ')
+            for i, name in enumerate(field_names):
+                buf.write('%s = ? ' % name)
+                if i != len(field_names) - 1:
+                    buf.write('AND ')
 
         buf.write('ORDER BY population DESC')
-        command = buf.getvalue()
-        self._cursor.execute(command, self._params)
-
-    def _generate_admin_codes(self) -> Iterator[str]:
-        """Generate the name/value pairs for constraining this collection."""
-        if self._parent.feature_code == 'PCLI':
-            max_level = 0
-        elif self._parent.feature_code == 'ADM1':
-            max_level = 1
-        elif self._parent.feature_code == 'ADM2':
-            max_level = 2
-        elif self._parent.feature_code == 'ADM3':
-            max_level = 3
-        elif self._parent.feature_code == 'ADM4':
-            max_level = 4
-
-        for level in (1, 2, 3, 4):
-            if level > max_level:
-                break
-            field_name = 'admin%d_code' % level
-            field_value = getattr(self._parent.geoname, field_name)
-            if field_value:
-                yield field_name, field_value
+        self._command = buf.getvalue()
 
     def __iter__(self):
         return self
 
-    def __next__(self) -> db.Geoname:
+    def __next__(self) -> Union['Geoname', 'State', 'City']:
+        if self._cursor is None:
+            self._cursor = db.CONN.cursor()
+            self._cursor.execute(self._command, self._params)
+
         result = next(self._cursor)
-        return db.Geoname(*result)
+        return _wrap(db.Geoname(*result))
 
     def __len__(self):
         cursor = db.CONN.cursor()
         command = self._command.replace('SELECT *', 'SELECT COUNT(*)')
         return cursor.execute(command, self._params)[0]
 
-
-class StateCollection(Collection):
-    def __init__(self, parent: Geoname, feature_code: str):
-        super().__init__(parent, 'A', feature_code)
-
-    def __next__(self):
-        #
-        # FIXME: use superclass
-        #
-        result = next(self._cursor)
-        return State(db.Geoname(*result))
-
     def __getitem__(self, key):
-        constraints = [code for (name, code) in self._generate_admin_codes()]
-        g = sc_list(key, country=self._parent.country_code, constraints=constraints)[0]
-        return State(g)
+        for g in db.select_geonames_name(key):
+            if self._parent is None or (self._parent != g and _contains(self._parent, g)):
+                return _wrap(g)
+        raise KeyError('could not find %r in %r' % (key, self))
 
-    def __contains__(self, key):
-        constraints = [code for (name, code) in self._generate_admin_codes()]
-        return bool(sc_list(key, country=self._parent.country_code, constraints=constraints))
-
-    def __str__(self):
-        return 'StateCollection(%r, %r)' % (self._parent, self._feature_code)
+    def __contains__(self, other):
+        if isinstance(other, str):
+            try:
+                self[other]
+            except KeyError:
+                return False
+            else:
+                return True
+        elif isinstance(other, Geoname):
+            if not self._parent:
+                return True
+            return _contains(self._parent, other.geoname)
+        elif isinstance(other, db.Geoname):
+            if not self._parent:
+                return True
+            return _contains(self._parent, other)
+        return False
 
     def __repr__(self):
-        return 'StateCollection(%r, %r)' % (self._parent, self._feature_code)
+        return 'Collection(%r, %r, %r)' % (
+            self._parent,
+            self._feature_class,
+            self._feature_code,
+        )
+
+
+G = Collection(None)
 
 
 class State(Geoname):
@@ -853,44 +933,7 @@ class State(Geoname):
 
     @property
     def cities(self):
-        return CityCollection(parent=self)
-
-
-class CityCollection(Collection):
-    def __init__(self, parent):
-        super().__init__(parent, 'P', None)
-
-    def __next__(self):
-        #
-        # FIXME: use superclass
-        #
-        result = next(self._cursor)
-        return City(db.Geoname(*result))
-
-    def __getitem__(self, key):
-        if isinstance(self._parent, Country):
-            return find_cities(key, country=self._parent.iso)[0]
-        else:
-            return find_cities(key, state=self._parent.name, country=self._parent.country_code)[0]
-
-    def __contains__(self, item):
-        if isinstance(self._parent, Country):
-            country_code = self._parent.iso
-        else:
-            country_code = self._parent.country_code
-
-        if isinstance(item, str):
-            for g in db.select_geonames_name(item):
-                if g.country_code == country_code:
-                    return True
-
-        return False
-
-    def __str__(self):
-        return 'CityCollection(%r)' % self._parent
-
-    def __repr__(self):
-        return 'CityCollection(%r)' % self._parent
+        return Collection(parent=self, feature_class='P')
 
 
 class City(Geoname):
